Package: HYPEtools
Version: 1.6.7
Title: Tools for Processing and Analyzing Files from the Hydrological
        Catchment Model HYPE
Description: Work with model files (setup, input, output) from
    the hydrological catchment model HYPE: Streamlined file import and export, standard 
    evaluation plot routines, diverse post-processing and aggregation routines 
    for hydrological model analysis. The HYPEtools package is also archived at <doi:10.5281/zenodo.7627955>
    and can be cited in publications with Brendel et al. (2024) <doi:10.1016/j.envsoft.2024.106094>.
Authors@R: c(person("Rene", "Capell", role = c("aut", "cre"), email = "hypetools.rene@smhi.se", comment = c(ORCID = "0000-0002-7784-1313")), 
    person("Conrad", "Brendel", role = "aut", email = "conrad.brendel@smhi.se", comment = c(ORCID = "0000-0002-5199-0580")),
    person("Jafet", "Andersson", role = "ctb"),
    person("David", "Gustafsson", role = "ctb"),
    person("Jude", list("Musuuza", "Lubega"), role = "ctb"))
Depends: R (>= 3.5.0)
Imports: clipr, colorspace, data.table (>= 1.9.8), dplyr, ggplot2,
        ggpubr, ggrepel, grDevices, graphics, lubridate, methods,
        ncdf4, parallel, patchwork, pbapply, purrr, rlang, scales,
        stats, stringr, tidyr, tidyselect, tools, utils, zoo
License: LGPL-3
URL: https://hypeweb.smhi.se/, https://github.com/rcapell/HYPEtools
BugReports: https://github.com/rcapell/HYPEtools/issues
RoxygenNote: 7.3.3
Encoding: UTF-8
Language: en-US
Suggests: rmarkdown, knitr, beepr, curl, DT, htmlwidgets, leaflet,
        leaflet.extras, mapview, plotly, rnaturalearth, sf, shiny,
        shinyalert, shinyFiles, shinyWidgets, terra, webshot
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-08 10:03:35 UTC; a002416
Author: Rene Capell [aut, cre] (ORCID: <https://orcid.org/0000-0002-7784-1313>),
  Conrad Brendel [aut] (ORCID: <https://orcid.org/0000-0002-5199-0580>),
  Jafet Andersson [ctb],
  David Gustafsson [ctb],
  Jude Musuuza [ctb],
  Jude Lubega [ctb]
Maintainer: Rene Capell <hypetools.rene@smhi.se>
Repository: CRAN
Date/Publication: 2026-04-08 19:30:14 UTC
Built: R 4.5.2; ; 2026-04-08 21:31:21 UTC; unix
