# Generated by roxygen2: do not edit by hand

S3method("[",HypeGeoData)
S3method("[",HypeMultiVar)
S3method("[",HypeSingleVar)
S3method("[",HypeXobs)
S3method(KGE,default)
S3method(NSE,HypeSingleVar)
S3method(NSE,default)
S3method(VE,default)
S3method(gof,default)
S3method(mae,default)
S3method(merge,HypeGeoData)
S3method(pbias,HypeSingleVar)
S3method(pbias,default)
S3method(print,summaryHypeGeoData)
S3method(print,summaryHypeSingleVar)
S3method(r,HypeSingleVar)
S3method(rPearson,default)
S3method(sKGE,default)
S3method(summary,HypeGeoData)
S3method(summary,HypeSingleVar)
S3method(valindex,default)
export("datetime<-")
export("hypeunit<-")
export("obsid<-")
export("outregid<-")
export("subid<-")
export("timestep<-")
export("variable<-")
export(AddInfoLine)
export(AllDownstreamSubids)
export(AllSimToPar)
export(AllUpstreamSubids)
export(AnnualRegime)
export(BarplotUpstreamClasses)
export(BestSimsToPar)
export(BoxplotSLCClasses)
export(CleanSLCClasses)
export(ColBlues)
export(ColDiffGeneric)
export(ColDiffTemp)
export(ColGreens)
export(ColNitr)
export(ColPhos)
export(ColPrec)
export(ColPurples)
export(ColQ)
export(ColReds)
export(ColTemp)
export(ColYOB)
export(CompareFiles)
export(ConvertDischarge)
export(CreateOptpar)
export(DirectUpstreamSubids)
export(EquallySpacedObs)
export(ExtractFreq)
export(ExtractStats)
export(GroupSLCClasses)
export(GwRetention)
export(HeadwaterSubids)
export(HypeGeoData)
export(HypeMultiVar)
export(HypeSingleVar)
export(HypeXobs)
export(IsHeadwater)
export(IsOutlet)
export(IsRegulated)
export(KGE)
export(MapRegionalSources)
export(MergeObs)
export(MergeXobs)
export(NSE)
export(OptimisedClasses)
export(OptimizedClasses)
export(OutletIds)
export(OutletNearObs)
export(OutletSubids)
export(PartyParrot)
export(PlotAnnualRegime)
export(PlotBasinOutput)
export(PlotBasinSummary)
export(PlotDurationCurve)
export(PlotEvaluationMaps)
export(PlotJohan)
export(PlotMapOutput)
export(PlotMapPoints)
export(PlotParValues)
export(PlotPerformanceByAttribute)
export(PlotSimObsRegime)
export(PlotSubbasinRouting)
export(ReadAllsim)
export(ReadAquiferData)
export(ReadBasinOutput)
export(ReadBranchData)
export(ReadClassData)
export(ReadCropData)
export(ReadDamData)
export(ReadDescription)
export(ReadFloodData)
export(ReadForcKey)
export(ReadGeoClass)
export(ReadGeoData)
export(ReadGlacierData)
export(ReadInfo)
export(ReadLakeData)
export(ReadMapOutput)
export(ReadMgmtData)
export(ReadObs)
export(ReadOptpar)
export(ReadOutregions)
export(ReadPTQobs)
export(ReadPar)
export(ReadPmsf)
export(ReadPointSourceData)
export(ReadSimass)
export(ReadSubass)
export(ReadTimeOutput)
export(ReadUpdate)
export(ReadWsOutput)
export(ReadXobs)
export(RemoveInfoLine)
export(RescaleSLCClasses)
export(RunHYPE)
export(ScaleAquiferData)
export(ScaleFloodData)
export(ScalePar)
export(SortGeoData)
export(SubidAttributeSummary)
export(SumSLCClasses)
export(SumUpstreamArea)
export(UpstreamGeoData)
export(UpstreamGroupSLCClasses)
export(UpstreamPointSources)
export(UpstreamSLCClasses)
export(VE)
export(VariableInfo)
export(VariableSearch)
export(VisualiseMapOutput)
export(VisualiseMapPoints)
export(VisualizeMapOutput)
export(VisualizeMapPoints)
export(WriteAquiferData)
export(WriteBasinOutput)
export(WriteBranchData)
export(WriteCropData)
export(WriteDamData)
export(WriteFloodData)
export(WriteForcKey)
export(WriteGeoClass)
export(WriteGeoData)
export(WriteGlacierData)
export(WriteHarmonizedData)
export(WriteHarmonizedSpatialDescription)
export(WriteInfo)
export(WriteLakeData)
export(WriteMapOutput)
export(WriteMgmtData)
export(WriteObs)
export(WriteOptpar)
export(WriteOutregions)
export(WritePTQobs)
export(WritePar)
export(WritePmsf)
export(WritePointSourceData)
export(WriteTimeOutput)
export(WriteXobs)
export(datetime)
export(distinctColorPalette)
export(gof)
export(hypeunit)
export(mae)
export(obsid)
export(outregid)
export(pbias)
export(r)
export(rPearson)
export(sKGE)
export(subid)
export(timestep)
export(valindex)
export(variable)
importFrom(clipr,clipr_available)
importFrom(clipr,write_clip)
importFrom(colorspace,LAB)
importFrom(colorspace,RGB)
importFrom(colorspace,darken)
importFrom(colorspace,hex)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,getDTthreads)
importFrom(data.table,is.data.table)
importFrom(data.table,setnames)
importFrom(data.table,transpose)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,if_any)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,right_join)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(dplyr,sym)
importFrom(dplyr,syms)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_discrete)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,waiver)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggpubr,colnames_style)
importFrom(ggpubr,get_legend)
importFrom(ggpubr,ggarrange)
importFrom(ggpubr,ggtexttable)
importFrom(ggpubr,tab_add_title)
importFrom(ggpubr,tbody_style)
importFrom(ggpubr,ttheme)
importFrom(grDevices,cairo_pdf)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.control)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.off)
importFrom(grDevices,hcl)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axTicks)
importFrom(graphics,axis)
importFrom(graphics,axis.POSIXct)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,boxplot)
importFrom(graphics,frame)
importFrom(graphics,grid)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(lubridate,as_datetime)
importFrom(methods,as)
importFrom(ncdf4,nc_close)
importFrom(ncdf4,nc_open)
importFrom(ncdf4,ncatt_get)
importFrom(ncdf4,ncvar_get)
importFrom(parallel,makePSOCKcluster)
importFrom(parallel,mclapply)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(pbapply,pbapply)
importFrom(pbapply,pblapply)
importFrom(pbapply,pbsapply)
importFrom(purrr,possibly)
importFrom(purrr,reduce)
importFrom(rlang,.data)
importFrom(scales,pseudo_log_trans)
importFrom(stats,aggregate)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,na.action)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,time)
importFrom(stats,weighted.mean)
importFrom(stringr,fixed)
importFrom(stringr,str_detect)
importFrom(stringr,str_starts)
importFrom(tidyr,drop_na)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,unnest)
importFrom(tidyselect,all_of)
importFrom(tidyselect,matches)
importFrom(tools,file_ext)
importFrom(tools,file_path_sans_ext)
importFrom(utils,glob2rx)
importFrom(utils,globalVariables)
importFrom(utils,modifyList)
importFrom(utils,read.table)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(utils,unzip)
importFrom(zoo,coredata)
